#!/bin/bash
#
# moOde OS Image Builder (C) 2017 Koda59
#
# This Program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This Program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 2017-12-15 TC adapted "Koda59 the script master" original script


#
# One Script To Rule Them All
#


VER="v1.0111"
DOWNLOAD_URL="http://moodeaudio.org/downloads/mos"
MOSBUILD_PROP="/home/pi/mosbuild/mosbuild.properties"
HOSTNAME="moode"
PASS="moodeaudio"

# check environment
[[ $EUID -ne 0 ]] && { echo "*** You must be root to run the script! ***" ; exit 1 ; } ;

# Wait a bit to be sure rc.local is not starting too fast
waitForRc () {
	TIMESTAMP=$(date)
	echo "** "$TIMESTAMP
	sleep $1
}

cancelBuild () {
	if [ $# -gt 0 ] ; then
		echo "$1"
	fi
	echo "** Image build cancelled"
	cd /home/pi
	rm -rf mosbuild 2> /dev/null
	rm -f *.zip 2> /dev/null
	exit 1
}

loadProperties () {
	local MOSBUILD_PROP=/home/pi/mosbuild/mosbuild.properties
	if [ -f $MOSBUILD_PROP ] ; then
		. $MOSBUILD_PROP
	else
		cancelBuild "** Error: unable to find properties file"
	fi
}

STEP_1 () {
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 1B - Change Password & download latest moOde release"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	local MOODE_REL_ZIP=`echo $MOODE_REL | awk -F"/" '{ print $NF }'`

	echo "** Change password for user pi to new password"
	echo "pi:$PASS" | chpasswd
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: password change failed"
	fi

	echo "** Download latest moOde release"
	wget -q $MOODE_REL
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: download failed"
	fi

	echo "** Unzip release"                           
	unzip -o -q $MOODE_REL_ZIP
	if [ $? -ne 0 ] ; then                                                                  
		cancelBuild "** Error: unzip failed"
	fi 
	rm -f $MOODE_REL_ZIP

	echo "** Install boot/config.txt"
	cp ./rel-stretch/boot/config.txt.default /boot/config.txt

	if [ $HDMI_DRIVE ]; then
		echo "** Patch  hdmi_drive=2"
		sed -i "s/hdmi_drive=2/hdmi_drive=$HDMI_DRIVE/" /boot/config.txt
	fi

}

STEP_2 () {
	waitForRc 10
	if [ -z "$UPDATE2MOODE" ] ; then
		STEP_1
		cd $MOSBUILD_DIR
		echo
		echo "////////////////////////////////////////////////////////////////"
		echo "//"
		echo "// STEP 2 - Expand the root partition to 3GB"
		echo "//"
		echo "////////////////////////////////////////////////////////////////"
		echo
	
		echo "** Expand SDCard to 3GB"
		cp ./rel-stretch/www/command/resizefs.sh ./
		chmod 0755 resizefs.sh
		sed -i "/PART_END=/c\PART_END=+3000M" ./resizefs.sh
		./resizefs.sh start
		rm -f resizefs.sh
	
		echo "** Cleanup"
		echo "** Reboot 1"
		echo "3A" > $MOSBUILD_STEP
		sync
		reboot
	fi
}

STEP_3A () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 3A - Install core packages"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Basic optimizations"
	dphys-swapfile swapoff
	update-rc.d dphys-swapfile remove
	rm -f /var/swap
	systemctl disable cron.service
	systemctl enable rpcbind
	DEBIAN_FRONTEND=noninteractive apt-get -y purge triggerhappy
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: unable to purge triggerhappy"
	fi

	if [ -z "$http_proxy" ] ; then
		echo "** No proxy configured"
	else
		echo "** Configuring proxy for Internet access"
		echo "Acquire::http::Proxy \"$http_proxy\";" > /etc/apt/apt.conf.d/10proxy
	fi

	echo "** Update Raspbian package list"
	DEBIAN_FRONTEND=noninteractive apt-get update
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: update failed"
	fi

	echo "** Upgrading Raspbian installed packages to latest available"
	DEBIAN_FRONTEND=noninteractive apt-get -y upgrade
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: upgrade failed"
	fi

	echo "** Reboot 2"
	echo "3B-4" > $MOSBUILD_STEP
	sync
	reboot
}

STEP_3B_4 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 3B - Install core packages"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Install core packages"
	DEBIAN_FRONTEND=noninteractive  apt-get -y install rpi-update php-fpm nginx sqlite3 php-sqlite3 memcached php-memcache mpc \
		bs2b-ladspa libbs2b0 libasound2-plugin-equal telnet automake sysstat squashfs-tools tcpdump shellinabox \
		samba smbclient udisks-glue ntfs-3g exfat-fuse git inotify-tools libav-tools avahi-utils

	if [ $? -ne 0 ] ; then
		cancelBuild "** Error:: Install failed"
	fi													 

	echo "** Disable shellinabox"
	systemctl disable shellinabox

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 4 - Install enhanced networking"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Install Host AP Mode packages"
	DEBIAN_FRONTEND=noninteractive apt-get -y install dnsmasq hostapd
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Disable hostapd and dnsmasq services"
	systemctl daemon-reload
	systemctl disable hostapd
	systemctl disable dnsmasq

	echo "** Install Bluetooth packages"
	DEBIAN_FRONTEND=noninteractive apt-get -y install bluez bluez-firmware pi-bluetooth \
		dh-autoreconf expect libortp-dev libbluetooth-dev libasound2-dev \
		libusb-dev libglib2.0-dev libudev-dev libical-dev libreadline-dev libsbc1 libsbc-dev
	
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Compile bluez-alsa"
	mkdir $MOSBUILD_DIR/work
	cd $MOSBUILD_DIR/work
	git clone https://github.com/Arkq/bluez-alsa.git
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Git clone failed"
	fi

	echo "Ignore warnings from autoreconf and configure"
	cd bluez-alsa
	autoreconf --install
	mkdir build
	cd build
	../configure --disable-hcitop --with-alsaplugindir=/usr/lib/arm-linux-gnueabihf/alsa-lib
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Configure failed"
	fi

	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi

	make install
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make install failed"
	fi

	cd $MOSBUILD_DIR
	rm -rf $MOSBUILD_DIR/work

	echo "** Verify bluealsa.service"
	if [ ! -f /lib/systemd/system/bluealsa.service ] ; then
		echo "** Creating bluealsa.service"
		echo "[Unit]" > /lib/systemd/system/bluealsa.service
		echo "Description=BluezAlsa proxy" >> /lib/systemd/system/bluealsa.service
		echo "Requires=bluetooth.service" >> /lib/systemd/system/bluealsa.service
		echo "After=bluetooth.service" >> /lib/systemd/system/bluealsa.service
		echo >> /lib/systemd/system/bluealsa.service
		echo "[Service]" >> /lib/systemd/system/bluealsa.service
		echo "Type=simple" >> /lib/systemd/system/bluealsa.service
		echo "User=bluealsa" >> /lib/systemd/system/bluealsa.service
		echo "Group=audio" >> /lib/systemd/system/bluealsa.service
		echo "ExecStart=/usr/bin/bluealsa" >> /lib/systemd/system/bluealsa.service
		echo >> /lib/systemd/system/bluealsa.service
		echo "[Install]" >> /lib/systemd/system/bluealsa.service
		echo "WantedBy=multi-user.target" >> /lib/systemd/system/bluealsa.service
		echo >> /lib/systemd/system/bluealsa.service
	fi

	echo "** Disable bluetooth services"
	systemctl daemon-reload
	systemctl disable bluetooth.service
	systemctl disable bluealsa.service
	systemctl disable hciuart.service
	mkdir -p /var/run/bluealsa
	sync

	echo "** Cleanup"
	DEBIAN_FRONTEND=noninteractive apt-get clean
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Cleanup failed"
	fi

	echo "** Reboot 3"
	echo "5-6" > $MOSBUILD_STEP
	sync
	reboot
}

STEP_5_6 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 5 - Install Rotary encoder driver"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Compile WiringPi"
	cp ./rel-stretch/other/wiringpi/wiringPi-2.44-96344ff.tar.gz ./

	tar xfz wiringPi-2.44-96344ff.tar.gz
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Un-tar failed"
	fi

	cd wiringPi-96344ff
	./build
	if [ $? -ne 0 ] ; then
		cancelBuild "** Compile failed"
	fi
	
	cd ..
	rm -rf wiringPi-96344ff
	rm -f wiringPi-2.44-96344ff.tar.gz

	echo "** Compile rotary encoder driver"
	cp rel-stretch/other/rotenc/rotenc.c ./
	gcc -std=c99 rotenc.c -orotenc -lwiringPi
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Compile failed"
	fi

	echo "** Cleanup"
	cp ./rotenc /usr/local/bin
	rm ./rotenc*

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 6 - Compile and install MPD"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo
	
	cd $MOSBUILD_DIR

	echo "** Create MPD runtime environment"
	useradd mpd
	mkdir /var/lib/mpd
	mkdir /var/lib/mpd/music
	mkdir /var/lib/mpd/playlists
	touch /var/lib/mpd/state
	chown -R mpd:audio /var/lib/mpd
	mkdir /var/log/mpd
	touch /var/log/mpd/log
	chmod 644 /var/log/mpd/log
	chown -R mpd:audio /var/log/mpd
	cp ./rel-stretch/mpd/mpd.conf.default /etc/mpd.conf
	chown mpd:audio /etc/mpd.conf
	chmod 0666 /etc/mpd.conf

	echo "** Install MPD devlibs"
	DEBIAN_FRONTEND=noninteractive apt-get -y install libmad0-dev libmpg123-dev libid3tag0-dev \
		libflac-dev libvorbis-dev libfaad-dev \
		libwavpack-dev libavcodec-dev libavformat-dev \
		libmp3lame-dev libsoxr-dev libcdio-paranoia-dev libiso9660-dev \
		libcurl4-gnutls-dev libasound2-dev libshout3-dev libyajl-dev \
		libmpdclient-dev libavahi-client-dev libsystemd-dev \
		libwrap0-dev libboost-dev libicu-dev libglib2.0-dev
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi 

	# Check for binary
	if [ -f ./rel-stretch/other/mpd/$MPD_BIN ] ; then
		echo "** Install pre-compiled binary"
		cp ./rel-stretch/other/mpd/$MPD_BIN /usr/local/bin/mpd

		echo "** Cleanup"
		DEBIAN_FRONTEND=noninteractive apt-get clean
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Cleanup failed"
		fi
	
		DEBIAN_FRONTEND=noninteractive apt-get autoremove
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Autoremove failed"
		fi
	else
		wget http://www.musicpd.org/download/mpd/0.20/mpd-0.20.12.tar.xz
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Download failed"
		fi
	
		tar xf mpd-0.20.12.tar.xz
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Un-tar failed"
		fi
	
		cd mpd-0.20.12
		sh autogen.sh
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Autogen failed"
		fi
	
		./configure --enable-database --enable-libmpdclient --enable-alsa \
			--enable-curl --enable-dsd --enable-ffmpeg --enable-flac \
			--enable-id3 --enable-soundcloud --enable-lame-encoder --enable-mad \
			--enable-mpg123 --enable-pipe-output --enable-recorder-output --enable-shout \
			--enable-vorbis --enable-wave-encoder --enable-wavpack --enable-httpd-output \
			--enable-soxr --with-zeroconf=avahi \
			--disable-bzip2 --disable-zzip --disable-fluidsynth --disable-gme \
			--disable-wildmidi --disable-sqlite --disable-jack --disable-ao --disable-oss \
			--disable-ipv6 --disable-pulse --disable-nfs --disable-smbclient \
			--disable-upnp --disable-expat --disable-lsr \
			--disable-sndfile --disable-audiofile --disable-sidplay
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Configure failed"
		fi
		
		make
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Make failed"
		fi
	
		make install
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Install failed"
		fi
	
		strip --strip-unneeded /usr/local/bin/mpd
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Strip failed"
		fi  
	
		echo "** Cleanup"
		cd $MOSBUILD_DIR
		rm -rf ./mpd-0.20.12
		DEBIAN_FRONTEND=noninteractive apt-get clean
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Cleanup failed"
		fi
	
		DEBIAN_FRONTEND=noninteractive apt-get autoremove
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Autoremove failed"
		fi
	fi

	echo "** Reboot 4"
	echo "7-8" > $MOSBUILD_STEP
	sync
	reboot  
}

STEP_7_8 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 7 - Create moOde runtime environment"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Create directories"
	mkdir /var/local/www
	mkdir /var/local/www/commandw
	mkdir /var/local/www/cssw
	mkdir /var/local/www/jsw
	mkdir /var/local/www/imagesw
	mkdir /var/local/www/imagesw/toggle
	mkdir /var/local/www/db
	mkdir /var/local/www/templatesw
	chmod -R 0755 /var/local/www
	mkdir /var/lib/mpd/music/RADIO

	echo "** Create mount points"
	mkdir /mnt/NAS
	mkdir /mnt/SDCARD
	mkdir /mnt/UPNP

	echo "** Create symlinks"
	ln -s /mnt/NAS /var/lib/mpd/music/NAS
	ln -s /mnt/SDCARD /var/lib/mpd/music/SDCARD
	ln -s /media /var/lib/mpd/music/USB
	ln -s /var/lib/mpd/music /var/www/vlmm90614385

	echo "** Create logfiles"
	touch /var/log/moode.log
	chmod 0666 /var/log/moode.log
	touch /var/log/php_errors.log
	chmod 0666 /var/log/php_errors.log

	echo "** Create misc files"
	cp ./rel-stretch/mpd/sticker.sql /var/lib/mpd
	cp -r "./rel-stretch/other/sdcard/Stereo Test/" /var/lib/mpd/music/SDCARD
	cp ./rel-stretch/network/interfaces.default /etc/network/interfaces
	cp ./rel-stretch/network/dhcpcd.conf.default /etc/dhcpcd.conf
	cp ./rel-stretch/network/hostapd.conf.default /etc/hostapd/hostapd.conf
	cp ./rel-stretch/var/local/www/db/moode-sqlite3.db.default /var/local/www/db/moode-sqlite3.db
	# if we are building over wifi, wpa_supplicant will already be configred
	# with ssid and pwd so we need to update cfg_network to match.
	if [ -z $SSID ] ; then
		cp ./rel-stretch/network/wpa_supplicant.conf.default /etc/wpa_supplicant/wpa_supplicant.conf
	else
		sqlite3 /var/local/www/db/moode-sqlite3.db "UPDATE cfg_network SET wlanssid='$SSID', wlanpwd='$PSK' WHERE id=2"
	fi
	sqlite3 /var/local/www/db/moode-sqlite3.db "UPDATE cfg_system SET value='$HOSTNAME' WHERE param='hostname'"
	echo "** Establish permissions"
	chmod 0777 /var/lib/mpd/music/RADIO
	chmod -R 0777 /var/local/www/db

	echo "** Misc deletes"
	rm -r /var/www/html
	rm /etc/update-motd.d/10-uname

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 8 - Install moOde sources and configs"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Install application sources and configs"
	rm /var/lib/mpd/music/RADIO/* 2> /dev/null
	rm /var/www/images/radio-logos/* 2> /dev/null
	cp ./rel-stretch/mpd/RADIO/* /var/lib/mpd/music/RADIO
	cp ./rel-stretch/mpd/playlists/* /var/lib/mpd/playlists
	cp -r ./rel-stretch/etc/* /etc
	cp -r ./rel-stretch/home/* /home/pi
	mv /home/pi/dircolors /home/pi/.dircolors
	mv /home/pi/xinitrc.default /home/pi/.xinitrc
	cp -r ./rel-stretch/lib/* /lib
	cp -r ./rel-stretch/usr/* /usr
	cp -r ./rel-stretch/var/* /var
	cp -r ./rel-stretch/www/* /var/www
	chmod 0755 /var/www/command/*
	/var/www/command/util.sh "emerald" "2ecc71" "27ae60"
	sqlite3 /var/local/www/db/moode-sqlite3.db "update cfg_system set value='Emerald' where param='themecolor'"

	echo "** Establish permissions for service files"
	echo "** MPD"
	chmod 0755 /etc/init.d/mpd
	chmod 0644 /lib/systemd/system/mpd.service
	chmod 0644 /lib/systemd/system/mpd.socket
	echo "** Bluetooth"
	chmod 0666 /etc/bluealsaaplay.conf
	chmod 0644 /etc/systemd/system/bluealsa-aplay@.service
	chmod 0644 /etc/systemd/system/bluealsa.service
	chmod 0644 /lib/systemd/system/bluetooth.service
	chmod 0755 /usr/local/bin/a2dp-autoconnect
	echo "** Rotenc"
	chmod 0644 /lib/systemd/system/rotenc.service
	echo "** Udev"
	chmod 0644 /etc/udev/rules.d/*

	echo "** Services are started by moOde Worker so lets disable them here"
	systemctl daemon-reload
	systemctl disable mpd.service
	systemctl disable mpd.socket
	systemctl disable rotenc.service

	echo "** Binaries will not have been installed yet, but let's disable the services here"
	chmod 0644 /lib/systemd/system/squeezelite-armv6l.service
	chmod 0644 /lib/systemd/system/squeezelite-armv7l.service
	systemctl disable squeezelite-armv6l
	systemctl disable squeezelite-armv7l
	chmod 0644 /lib/systemd/system/upmpdcli.service
	systemctl disable upmpdcli.service

	echo "** Initial permissions for certain files. These also get set during moOde Worker startup"
	chmod 0777 /var/local/www/playhistory.log
	chmod 0777 /var/local/www/currentsong.txt
	touch /var/local/www/libcache.json
	chmod 0777 /var/local/www/libcache.json

	echo "** Re-establish image build autorun in rc.local"
	sed -i "s/^exit.*//" /etc/rc.local
	echo "$MOSBUILD_DIR/moosimbu.sh >> /home/pi/mosbuild.log 2>> /home/pi/mosbuild.log" >> /etc/rc.local
	echo "exit 0" >> /etc/rc.local

	echo "** Update sudoers file"
	cp /etc/sudoers /tmp/sudoers.bak
	chmod 0777 /tmp/sudoers.bak
	echo -e "pi\tALL=(ALL) NOPASSWD: ALL" >> /tmp/sudoers.bak
	echo -e "www-data\tALL=(ALL) NOPASSWD: ALL" >> /tmp/sudoers.bak
	visudo -cf /tmp/sudoers.bak
	if [ $? -eq 0 ]; then
		cp /tmp/sudoers.bak /etc/sudoers
	else
		cancelBuild "** Error: Update sudoers file failed"
	fi
	
	echo "** Reboot 5"
	echo "9-10" > $MOSBUILD_STEP
	sync
	reboot
}

STEP_9_10 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 9 - Alsaequal"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Install alsaequal" 
	amixer -D alsaequal > /dev/null

	echo "** Establish permissions"
	chmod 0755 /usr/local/bin/alsaequal.bin
	chown mpd:audio /usr/local/bin/alsaequal.bin
	rm /usr/share/alsa/alsa.conf.d/equal.conf

	echo "** Wait 45 secs for moOde Startup to complete"
	sleep 45

	echo "** List MPD outputs"
	mpc outputs
	echo "** Enable only output 1"
	mpc enable only 1

	echo "** Alsaequal installed"

	if [ -z "$SQUASH_FS" ] ; then
		echo "** STEP 10 - Squashfs option not selected"
		STEP_11
	else
		echo
		echo "////////////////////////////////////////////////////////////////"
		echo "//"
		echo "// STEP 10 - Optionally squash /var/www"
		echo "//"
		echo "////////////////////////////////////////////////////////////////"
		echo
	
		echo "** Add squashfs mount to /etc/fstab"
		echo "/var/local/moode.sqsh   /var/www        squashfs        ro,defaults     0       0" >> /etc/fstab

		echo "** Squash /var/www"
		rm /var/local/moode.sqsh
		mksquashfs /var/www /var/local/moode.sqsh

		echo "** Remove contents of /var/www"
		rm -rf /var/www/*
	fi

	echo "** Reboot 6"
	echo "11" > $MOSBUILD_STEP
	sync
	reboot
}

STEP_11 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	if [ -z "$LATEST_KERNEL" ] ; then
		echo "** STEP 11 - Latest kernel option not selected"
		STEP_12_13
	else
		echo
		echo "////////////////////////////////////////////////////////////////"
		echo "//"
		echo "// STEP 11 - Optionally, install latest Linux Kernel"
		echo "//"
		echo "////////////////////////////////////////////////////////////////"
		echo

		echo "** Download and installing latest Linux kernel"
		PRUNE_MODULES=1 rpi-update
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Rpi-update failed"
		else
			echo "** Cleanup"
		    rm -rf /lib/modules.bak
		    rm -rf /boot.bak
			echo "** Reboot 7"
			echo "12-13" > $MOSBUILD_STEP
			sync
			reboot
		fi
	fi
}

STEP_12_13 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 12 - Launch and configure moOde!"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "1. Initial configuration"
	echo
	echo "a. http://moode"
	echo "b. Browse Tab, Default Playlist, Add"
	echo "c. Menu, Configure, Sources, UPDATE mpd database"
	echo "d. Menu, Audio, Mpd options, EDIT SETTINGS, APPLY"
	echo "e. Menu, System, Set timezone"
	echo "f. Clear system logs, YES"
	echo "g. Compact sqlite database, YES"
	echo "h. Keyboard"
	echo "i. Layout"
	echo
	echo "2. Verification"
	echo
	echo "a) Playback tab"
	echo "b) Scroll to the last item which should be the Stereo Test track"
	echo "c) Click to begin play"
	echo "d) Menu, Audio info"
	echo "e) Verify Output stream is 16 bit 48 kHz"
	echo

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 13 - Final prep for image"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "1. Optionally check the boot partition."
	echo
	echo "NOTE: Run these commands one at a time."
	echo
	echo "If the message "There are differences between boot sector and its backup" appears,"
	echo "enter 1 'Copy original to backup', then y to 'Perform changes ?'"
	echo
	echo "sudo umount /boot"
	echo "sudo dosfsck -tawl /dev/mmcblk0p1"
	echo "sudo dosfsck -r /dev/mmcblk0p1"
	echo "sudo dosfsck -V /dev/mmcblk0p1"
	echo "sudo mount /boot"
	echo

	echo "** Clear system logs"
	/var/www/command/util.sh clear-syslogs

	echo "** Remove DHCP lease files "
	rm /var/lib/dhcpcd5/*

	echo "** Reset network config to defaults "
	cp ./rel-stretch/network/interfaces.default /etc/network/interfaces
#	cp ./rel-stretch/network/wpa_supplicant.conf.default /etc/wpa_supplicant/wpa_supplicant.conf
	cp ./rel-stretch/network/dhcpcd.conf.default /etc/dhcpcd.conf
	cp ./rel-stretch/network/hostapd.conf.default /etc/hostapd/hostapd.conf

	if [ -z "$ADDL_COMPONENTS" ] ; then
		echo "** Additional components option not selected"
		finalCleanup
	else
		echo "** Reboot 8"
		echo "C1_C9" > $MOSBUILD_STEP
		echo "** Reboot"
		sync
		reboot
	fi
 }

COMP_C1_C9 () {
	waitForRc 10
	cd $MOSBUILD_DIR

	echo
	echo "################################################################"
	echo "#"
	echo "#"
	echo "# Install additional components"
	echo "#"
	echo "#"
	echo "################################################################"
	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 1 - MiniDLNA"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Install MiniDLNA package"
	DEBIAN_FRONTEND=noninteractive apt-get -y -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install minidlna
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Disable MiniDLNA service"
	systemctl disable minidlna
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Disable failed"
	fi

	echo "** Install Djmount package"
	DEBIAN_FRONTEND=noninteractive apt-get -y install djmount
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 2 - Auto-shuffle"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Build Auto-shuffle"
	git clone https://github.com/Joshkunz/ashuffle.git
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Git clone failed"
	fi

	cd $MOSBUILD_DIR/ashuffle
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi

	echo "** Install binary"
	cd ..
	cp ./ashuffle/ashuffle /usr/local/bin
	rm -rf ./ashuffle

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 3 - MPD Audio Scrobbler"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Build MPDAS"
	git clone https://github.com/hrkfdn/mpdas
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Git clone failed"
	fi

	cd $MOSBUILD_DIR/mpdas
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi

	echo "** Install binary"
	cp ./mpdas /usr/local/bin
	cd ..
	rm -rf ./mpdas

	echo "** Install conf file"
	cp ./rel-stretch/usr/local/etc/mpdasrc.default /usr/local/etc/mpdasrc
	chmod 0755 /usr/local/etc/mpdasrc

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 4 - Shairport-sync"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Install shairport-sync devlibs"
	DEBIAN_FRONTEND=noninteractive apt-get -y install autoconf libtool libdaemon-dev libasound2-dev libpopt-dev libconfig-dev \
		avahi-daemon libavahi-client-dev libssl-dev libsoxr-dev
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	# Check for binary
	if [ -f ./rel-stretch/other/shairport-sync/$SPS_BIN ] ; then
		echo "** Install pre-compiled binary"
		cp ./rel-stretch/other/shairport-sync/$SPS_BIN /usr/local/bin/shairport-sync
	else
		echo "** Compile shairport-sync"
		git clone https://github.com/mikebrady/shairport-sync.git
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Git clone failed"
		fi
		
		cd shairport-sync
		autoreconf -i -f
		./configure --with-alsa --with-avahi --with-ssl=openssl --with-soxr --with-metadata --with-stdout --with-systemd
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Configure failed"
		fi
	
		make
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Make failed"
		fi
	
		make install
		if [ $? -ne 0 ] ; then
			cancelBuild "** Error: Install failed"
		fi
	
		echo "** Disable shairport-sync service"
		systemctl disable shairport-sync
	fi
	
	echo "** Install conf file"
	cd ..
	cp ./rel-stretch/usr/local/etc/shairport-sync.conf /usr/local/etc
	rm -rf ./shairport-sync

	echo "** Cleanup"
	DEBIAN_FRONTEND=noninteractive apt-get clean
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Cleanup failed"
	fi

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 5 - Squeezelite"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Install pre-compiled binaries"
	echo "** Install armv6l bin"
	cp ./rel-stretch/other/squeezelite/$SL_BIN6 /usr/local/bin/squeezelite-armv6l
	echo "** Install armv7l bin"
	cp ./rel-stretch/other/squeezelite/$SL_BIN7 /usr/local/bin/squeezelite-armv7l

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 6 - Upmpdcli"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Install upmpdcli devlibs"
	DEBIAN_FRONTEND=noninteractive apt-get -y install libmicrohttpd-dev libexpat1-dev \
	libxml2-dev libxslt1-dev libjsoncpp-dev python-requests python-pip
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Compile Libupnp jfd5"
	cp ./rel-stretch/other/upmpdcli/libupnp-1.6.20.jfd5.tar.gz ./
	tar xfz ./libupnp-1.6.20.jfd5.tar.gz
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Un-tar failed"
	fi
	cd libupnp-1.6.20.jfd5
	./configure --prefix=/usr --sysconfdir=/etc
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Configure failed"
	fi
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi
	make install
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi
	echo "** Cleanup"
	cd ..
	rm -rf ./libupnp-1.6.20.jfd5
	rm libupnp-1.6.20.jfd5.tar.gz
	

	echo "** Compile Libupnpp"	
	cp ./rel-stretch/other/upmpdcli/libupnpp-0.16.0.tar.gz ./
	tar xfz ./libupnpp-0.16.0.tar.gz
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Un-tar failed"
	fi
	cd libupnpp-0.16.0
	./configure --prefix=/usr --sysconfdir=/etc
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Configure failed"
	fi
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi
	make install
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi
	echo "** Cleanup"
	cd ..
	rm -rf ./libupnpp-0.16.0
	rm libupnpp-0.16.0.tar.gz
		
	echo "** Compile Upmpdcli"	
	cp ./rel-stretch/other/upmpdcli/upmpdcli-1.2.15.tar.gz ./
	tar xfz ./upmpdcli-1.2.15.tar.gz 
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Un-tar failed"
	fi
	cd upmpdcli-1.2.15
	./configure --prefix=/usr --sysconfdir=/etc
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Configure failed"
	fi
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi
	make install
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi
	echo "** Cleanup"
	cd ..
	rm -rf ./upmpdcli-1.2.15
	rm upmpdcli-1.2.15.tar.gz
	
	echo "** Configure runtime env"
	useradd upmpdcli
	cp ./rel-stretch/lib/systemd/system/upmpdcli.service /lib/systemd/system
	cp ./rel-stretch/etc/upmpdcli.conf /etc
	chmod 0644 /etc/upmpdcli.conf
	systemctl daemon-reload
	systemctl disable upmpdcli
	
	echo "** Compile Upexplorer"	
	git clone https://@opensourceprojects.eu/git/p/libupnppsamples/code libupnppsamples-code
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Git clone failed"
	fi
	cd libupnppsamples-code
	./autogen.sh
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Autogen failed"
	fi
	./configure
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Configure failed"
	fi
	make
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Make failed"
	fi
	make install
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Cleanup"
	cd ..
	rm -rf ./libupnppsamples-code
	DEBIAN_FRONTEND=noninteractive apt-get clean
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Cleanup failed"
	fi

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 7 - Optionally install gmusicapi"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "This component enables access to Google Play Music service via UPnP renderer."
	echo "If its not installed, the Google Play section in UPnP config screen will not be present."
	echo "sudo pip install gmusicapi"

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 8 - Local UI display"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Install Local UI packages"
	DEBIAN_FRONTEND=noninteractive apt-get -y install xinit xorg lsb-release xserver-xorg-legacy chromium-browser libgtk-3-0
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Install failed"
	fi

	echo "** Permissions and service config"
	sed -i "s/allowed_users=console/allowed_users=anybody/" /etc/X11/Xwrapper.config
	systemctl daemon-reload
	systemctl disable localui

	echo "** Cleanup"
	DEBIAN_FRONTEND=noninteractive apt-get clean
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Cleanup failed"
	fi

	echo
	echo "Configure Chrome Browser"
	echo 
	echo "NOTE: These steps are performed AFTER actually starting local display via System config,"
	echo "rebooting and then accessing moOde on the local display."
	echo
	echo "a. Connect a keyboard."
	echo "b. Press Ctrl-t to open a separate instance of Chrome Browser."
	echo "c. For Raspberry Pi 7" Touch Display open Chrome settings and set the zoom to 75%.""
	echo "d. Optionally, enter url chome://extensions and install the xontab virtual keyboard extension."
	echo "e. Enter url chrome://flags and scroll down to Overlay Scrollbars and enable the setting."
	echo

	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// COMPONENT 9 - Allo Piano 2.1 Firmware"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "Download and install firmware"
	wget https://github.com/allocom/piano-firmware/archive/master.zip
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Download failed"
	fi
	unzip -q master.zip 
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Unzip failed"
	fi

	echo "Cleanup"
	cp -r ./piano-firmware-master/lib/firmware/allo /lib/firmware
	rm ./master.zip
	rm -rf ./piano-firmware-master

	finalCleanup
}

finalCleanup () {
	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// Final cleanup"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	cd $MOSBUILD_DIR

	echo "** Remove mosbuild"
	cp ./rel-stretch/etc/rc.local /etc/rc.local
	cd ..
	rm -rf $MOSBUILD_DIR
	echo "** Clean package cache"
	DEBIAN_FRONTEND=noninteractive apt-get clean
	if [ $? -ne 0 ] ; then
		cancelBuild "** Error: Cleanup failed"
	fi
	echo "** Clear syslogs"
	/var/www/command/util.sh clear-syslogs
	STOP_TIME=$(date +%s)
	INSTALLING_TIME=$(($STOP_TIME - $START_TIME))
	INSTALLING_TIME=$(date -u -d @$INSTALLING_TIME +"%T")
	TIMESTAMP=$(date)
	echo "** Installation time : $INSTALLING_TIME"
	echo "** "$TIMESTAMP
	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "// END"
	echo "////////////////////////////////////////////////////////////////"
	echo

	echo "** Final reboot"
#	aplay /usr/share/sounds/alsa/Front_Center.wav
	reboot
}





##//////////////////////////////////////////////////////////////
##
## FIRST STEP !
##
##//////////////////////////////////////////////////////////////


readYnInput () {
	while true; do
	    read -p "$1" YN
	    case $YN in
	        [y] ) break;;
	        [n] ) break;;
	        * ) echo "** Valid entries are y|n";;
	    esac
	done
}

readStrInput () {
    read -p "$1" STR
	readYnInput "** Make corrections (y/n)? "
	if [ $YN = "y" ] ; then
		readStrInput "$1"
	fi
}

cancelPrep () {
	if [ $# -gt 0 ] ; then
		echo "$1"
	fi
	echo "** Image build cancelled"
	ls mosbuild/*.img > /dev/null 2>&1
	# if no image files present remove the dir
	if [ $? -ne 0 ] ; then
		rm -rf ./mosbuild 2> /dev/null
	fi
	cleanUp
	exit 1
}

mainBanner () {
	echo "****************************************************************"
	echo "**"
	echo "**  Moode OS Image Builder $VER"
	echo "**"
	echo "**  Welcome to the automated process for creating the wonderful"
	echo "**  custom Linux OS that runs moOde audio player."
	echo "**"
	echo "**  You will need a Raspberry Pi running Raspbian with SSH"
	echo "**  enabled, at least 2.5GB free space on the boot SDCard and"
	echo "**  a spare USB or USB-SDCard drive that the new OS will be"
	echo "**  written to during the build process."
	echo "**"
	echo "**  Be sure to backup the SDCard used to boot your Pi"
	echo "**"
	echo "****************************************************************"
	echo
	echo "////////////////////////////////////////////////////////////////"
	echo "//"
	echo "// STEP 1 - Download Raspbian Lite and create a new, base image"
	echo "//"
	echo "////////////////////////////////////////////////////////////////"
	echo

	readYnInput "** Do you have a backup of your boot SDCard (y/n)? "
	if [ $YN = "n" ] ; then
		cancelPrep
	fi
	echo "** Unplug all USB storage devices from the Pi"
	readYnInput "** Are all USB storage devices unplugged (y/n)? "
	if [ $YN = "n" ] ; then
		cancelPrep
	fi
}

testDiskSpace () {
	echo "** Check free disk space"
	FREESPACE="$(df -k . --output=avail | tail -1)"
	if [ $FREESPACE -lt 2500000 ] ; then
		cancelPrep "** Error: Not enough free space on boot SDCard: 2.5GB required"
	else
		mkdir -p mosbuild 2> /dev/null
	fi
}

testUnzip () {
	if [ ! -f /usr/bin/unzip ] ; then
		echo "** Install Unzip utility"
		apt-get install unzip
		if [ $? -ne 0 ] ; then
			cancelPrep "** Error: Install failed"
		fi
	fi
}

getTargetUsb () {
	ls -tr /dev/disk/by-id 2> /dev/null | sort > mosbuild_befor_usb.txt
	echo "** Plug in target USB drive for the new OS"
	readYnInput "** Is target USB drive plugged in (y/n)? "
	if [ $YN = "n" ] ; then
		cancelPrep
	fi

	sleep 2
	ls -tr /dev/disk/by-id 2> /dev/null | sort > mosbuild_after_usb.txt
	DEVICE=`comm -13 mosbuild_befor_usb.txt mosbuild_after_usb.txt`	
	if [ -z "$DEVICE" ] ; then
		cancelPrep "** Error: Unable to find USB drive"
	fi

	# get device and partitions if any
	USBDEV=`comm -13 mosbuild_befor_usb.txt mosbuild_after_usb.txt | grep -v part`
	USBPART=`comm -13 mosbuild_befor_usb.txt mosbuild_after_usb.txt | grep part`
	rm -f mosbuild_befor_usb.txt mosbuild_after_usb.txt

	# identify each partition for umounting later
	if [ ! -z "$USBPART" ] ; then
		cd /dev/disk/by-id
		DEVPARTS=""
		for i in $USBPART ; do
			PARTDEV=`readlink -f $i`
			DEVPARTS="${DEVPARTS}${PARTDEV} "
		done
		cd - > /dev/null 2>&1
	fi

	if [ -z "$USBDEV" ] ; then
		# its odd to have no device so lets try to derive it from a partition
		USBDEV=`echo $DEVPARTS | awk -F" " '{print $1}' | head -c -2`
	else
		USBDEV=`cd /dev/disk/by-id ; readlink -f $USBDEV ; cd - > /dev/null 2>&1`
	fi

	echo "** USB drive detected on $USBDEV"
	# unmount partitions if any exist on the dev and are already mounted
	if [ ! -z "$USBPART" ] ; then
		for PART in $DEVPARTS ; do
			TRIES=0
			findmnt $PART > /dev/null
			if [ $? -eq 0 ] ; then
				umount $PART > /dev/null 2>&1
				while [ $? -ne 0 ] && [ $TRIES -lt 3 ] ; do
					let TRIES++
					echo "** Partition $PART cannot be unmounted, try $TRIES-3"
					sleep 5
					umount -l $PART > /dev/null 2>&1
				done
			fi
		done

		if [ $? -ne 0 ] ; then
			cancelPrep
		else
			echo "** Partitions unmounted on $USBDEV"
		fi
	fi
}

getOptions () {
	NUMOPT=5
	IDXOPT=1
	proxyServer
	useWireless
	squashFs
	latestKernel
	addlComponents
}

confirmBuild () {
	echo "** Ready for automated image build"
	readYnInput "** Proceed (y/n)? "
	if [ $YN = "n" ] ; then
		cancelPrep
	fi
}

#
# BEGIN OPTIONS
#

changePwd () {
	echo "** Enter your new password / leave blank to keep default password: moodeaudio"
	readStrInput "** New Password: "
	if [ $STR ] ; then
		PASS="$STR"
	fi
}

changeHostname () {
	echo "** Enter the new hostname / leave blank to keep default hostname: moode"
	echo "** Length 3-63 chars, A-Z, a-z, 0-9, and hyphen (-) are allowed."
	readStrInput "** New Hostname: "
	if [ $STR ] ; then
		HOSTNAME="$STR"
	fi
}

passNhostname () {
	readYnInput "** Change the default password and hostname (moodeaudio, moode) (y/n)"
	if [ $YN = "y" ] ; then
		changePwd
		changeHostname
	fi
}

changeHDMI (){
	readYnInput "** Change the hdmi_drive option (default = 2) (y/n)"
	if [ $YN = "y" ] ; then
		echo "set to 0 or 1 or 2"
		readStrInput "** hdmi_drive= "
		if [ $STR ] ; then
			HDMI_DRIVE="$STR"
		fi
	fi

}

proxyServer () {
	readYnInput "** Option $((IDXOPT++))-$NUMOPT: use a proxy server for Internet access (y/n)? "
	if [ $YN = "y" ] ; then
		echo "** Enter proxy url in the format http://[[username]:[password]@]<proxy address>:<proxy port>/"
		readStrInput "** Proxy url: "
		HTTP_PROXY="$STR"
		HTTPS_PROXY="$STR"
	fi
}

configWireless() {
	readStrInput "** SSID: "
	SSID="$STR"
	readStrInput "** Password: "
	PSK="$STR"
	readStrInput "** Country (GB, FR, US, ES,...): "
	COUNTRY="$STR"
}

getWirelessInfos () {
	if isConnected && [ -f /etc/wpa_supplicant/wpa_supplicant.conf ] ; then
		COUNTRY=`awk '(/^/||/ /) && /country=/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/country=//'`
		SSID=`awk '(/^/||/ /) && /ssid=\"/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/ssid=\"\(.*\)\"/\1/'`
		PSK=`awk '(/^/||/ /) && /psk=\"/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/psk=\"\(.*\)\"/\1/'`
	fi
}

useWireless () {
	readYnInput "** Option $((IDXOPT++))-$NUMOPT: use a WiFi connection instead of Ethernet (y/n)? "
	if [ $YN = "y" ] ; then
		if isConnected && [ -f /etc/wpa_supplicant/wpa_supplicant.conf ] ; then
			COUNTRY=`awk '(/^/||/ /) && /country=/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/country=//'`
			SSID=`awk '(/^/||/ /) && /ssid=\"/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/ssid=\"\(.*\)\"/\1/'`
			PSK=`awk '(/^/||/ /) && /psk=\"/' /etc/wpa_supplicant/wpa_supplicant.conf | sed -e 's/^[ \t]*//;s/psk=\"\(.*\)\"/\1/'`
			echo "** You're already connected in wifi with theses settings :"
			echo "** country : $COUNTRY"
			echo "** SSID : $SSID"
			echo "** Password : $PSK"
			readYnInput "** Theses settings are correct (y/n)? "
			if [ $YN = "n" ] ; then
				configWireless
			fi
		else
			configWireless
		fi
	fi
}

squashFs () {
	readYnInput "** Option $((IDXOPT++))-$NUMOPT: configure /var/www as squashfs (y/n)? "
	if [ $YN = "y" ] ; then
		SQUASH_FS=$YN
	fi
}

latestKernel () {
	readYnInput "** Option $((IDXOPT++))-$NUMOPT: install latest Linux Kernel (y/n)? "
	if [ $YN = "y" ] ; then
		LATEST_KERNEL=$YN
	fi
}

addlComponents () {
	echo "** Option $((IDXOPT++))-$NUMOPT: Airplay, Ashuffle, LocalUI, Scrobbler, Squeezelite and UPnP/DLNA"
	readYnInput "** Install additional components (y/n)? "
	if [ $YN = "y" ] ; then
		ADDL_COMPONENTS=$YN
	fi
}

#
# END OPTIONS
#

testInternet () {
	echo "** Test Internet connection"
	if [ ! -z "$HTTP_PROXY" ] ; then
		export http_proxy=$HTTP_PROXY
	fi
	wget -q http://www.google.com
	if [ $? -eq 0 ] ; then
		rm -f index.html
	else
		if [ ! -z "$HTTP_PROXY" ] ; then
			echo "** Proxy url: $HTTP_PROXY"
		fi
		cancelPrep "** Error: Unable to detect Internet connection"
	fi
}

dnldHelpers () {
	echo "** Download helper files"
	wget -q $DOWNLOAD_URL/mosbuild.properties -O mosbuild.properties
	if [ $? -ne 0 ] ; then
		cancelPrep "** Error: Unable to download Properties file"
	fi
}

updProperties () {
	echo "** Add options to properties file"
	echo "START_TIME=$(date +%s)" >> mosbuild.properties
	echo "PASS=\"$PASS\"" >> mosbuild.properties
	echo "HOSTNAME=\"$HOSTNAME\"" >> mosbuild.properties
	if [ ! -z "$HDMI_DRIVE" ] ; then
		echo "HDMI_DRIVE=$HDMI_DRIVE" >> mosbuild.properties
	fi
	if [ ! -z "$UPDATE2MOODE" ] ; then
		echo "UPDATE2MOODE=$UPDATE2MOODE" >> mosbuild.properties
	fi
	if [ ! -z "$HTTP_PROXY" ] ; then
		echo "export http_proxy=$HTTP_PROXY" >> mosbuild.properties
		echo "export https_proxy=$HTTPS_PROXY" >> mosbuild.properties
	fi
	if [ ! -z "$SSID" ] ; then
		echo "SSID=\"$SSID\"" >> mosbuild.properties
		echo "PSK=\"$PSK\"" >> mosbuild.properties
		echo "COUNTRY=\"$COUNTRY\"" >> mosbuild.properties
	fi
	if [ ! -z "$SQUASH_FS" ] ; then
		echo "SQUASH_FS=$SQUASH_FS" >> mosbuild.properties
	fi
	if [ ! -z "$LATEST_KERNEL" ] ; then
		echo "LATEST_KERNEL=$LATEST_KERNEL" >> mosbuild.properties
	fi
	if [ ! -z "$ADDL_COMPONENTS" ] ; then
		echo "ADDL_COMPONENTS=$ADDL_COMPONENTS" >> mosbuild.properties
	fi
}

loadEnv () {
	echo "** Load properties into env"
	local MOSBUILD_PROP=mosbuild.properties
	if [ -f $MOSBUILD_PROP ] ; then
		. $MOSBUILD_PROP
	else
		cancelPrep "** Error: Unable to find properties file"
	fi
}

dnldRaspbian () {
	local RASPBIAN_ZIP=`echo $RASPBIAN_DNLD | awk -F"/" '{ print $NF }'`
	echo "** Download Rasbian Stretch Lite $RASPBIAN_ZIP"  
	cd mosbuild
	wget -q --show-progress $RASPBIAN_DNLD -O $RASPBIAN_ZIP
	if [ $? -ne 0 ] ; then
		cancelPrep "** Error: Download failed"
	fi
	cd ..
}

unzipRaspbian () {
	local RASPBIAN_ZIP=`echo $RASPBIAN_DNLD | awk -F"/" '{ print $NF }'`
	echo "** Unzip $RASPBIAN_ZIP"
	cd mosbuild
	unzip -o $RASPBIAN_ZIP
	if [ $? -ne 0 ] ; then
		rm -f $RASPBIAN_ZIP
		cancelPrep "** Error: Unzip failed"
	else
		rm -f $RASPBIAN_ZIP
	fi
	cd ..
}

mountImage () {
	echo "** Mount Raspbian image partitions"
	mkdir part1
	mkdir part2
	LOOPDEV=$(sudo losetup -f)
	losetup -P $LOOPDEV mosbuild/$RASPBIAN_IMG
	mount -t vfat "$LOOPDEV"p1 part1
	if [ $? -ne 0 ] ; then
		rmdir part1
		rmdir part2
		cancelPrep "** Error: Mount failed for partition 1"
	fi

	mount -t ext4 "$LOOPDEV"p2 part2
	if [ $? -ne 0 ] ; then
		umount part1
		rmdir part1
		rmdir part2
		cancelPrep "** Error: Mount failed for partition 2"
	fi
}

modifyImage () {
	echo "** Modify image"
	touch part1/ssh
	echo "** Enable SSH"

	if [ ! -z "$SSID" ] ; then
		echo "#########################################" > part1/wpa_supplicant.conf
		echo "# This file is automatically generated by" >> part1/wpa_supplicant.conf
		echo "# the player Network configuration page." >> part1/wpa_supplicant.conf
		echo "#########################################" >> part1/wpa_supplicant.conf
		echo >> part1/wpa_supplicant.conf
		echo "country=$COUNTRY" > part1/wpa_supplicant.conf
		echo "ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev" >> part1/wpa_supplicant.conf
		echo "update_config=1" >> part1/wpa_supplicant.conf
		echo >> part1/wpa_supplicant.conf
		echo "network={" >> part1/wpa_supplicant.conf
		echo "ssid=\"$SSID\"" >> part1/wpa_supplicant.conf
		echo "scan_ssid=1" >> part1/wpa_supplicant.conf
		echo "psk=\"$PSK\"" >> part1/wpa_supplicant.conf
		echo "}" >> part1/wpa_supplicant.conf
	fi

	sed -i "s/init=.*//" part1/cmdline.txt
	sed -i "s/quiet.*//" part1/cmdline.txt
	rm part2/etc/init.d/resize2fs_once
	rm part2/etc/rc3.d/S01resize2fs_once
	echo "** Remove auto-resize task"

	sed -i "s/^/net.ifnames=0 /" part1/cmdline.txt
	echo "** Enable familiar network interface names"

	mkdir part2$MOSBUILD_DIR
	cp -f mosbuild.properties part2$MOSBUILD_DIR
	cp -f /home/pi/moosimbu.sh part2$MOSBUILD_DIR
	echo "2" >> part2$MOSBUILD_STEP
	chmod +x part2$MOSBUILD_DIR/moosimbu.sh
	chown -R 1000.1000 part2$MOSBUILD_DIR
	echo "** Install main worker script"

	sed -i "s/^exit.*//" part2/etc/rc.local
	echo "$MOSBUILD_DIR/moosimbu.sh >> /home/pi/mosbuild.log 2>> /home/pi/mosbuild.log" >> part2/etc/rc.local
	echo "exit 0" >> part2/etc/rc.local
	echo "** Enable script for autorun after reboot"

	sed -i "s/raspberrypi/$HOSTNAME/" part2/etc/hostname
	sed -i "s/raspberrypi/$HOSTNAME/" part2/etc/hosts
	cp /etc/fake-hwclock.data part2/etc/ 2> /dev/null
	echo "** Change host name to $HOSTNAME"
	echo "** Flush cached disk writes"
	sync
}

umountImage () {
	echo "** Image unmounted"
	losetup -D
	umount part1
	umount part2
	rmdir part1
	rmdir part2
}

writeImage () {
	echo "** Write image to USB drive on $USBDEV"
	dd if=mosbuild/$RASPBIAN_IMG of=$USBDEV
	echo "** Flush cached disk writes"
	sync
	if [ $? -eq 0 ] ; then
		echo "**"
		echo "** New base OS image created"
		echo "**"
		echo "** Remove the USB drive and use it to boot a Raspberry Pi"
		echo "** The build will automatically continue at STEP 2 after boot"
		echo "**"
		readYnInput "** Save base OS img for additional builds (y/n)? "
		if [ $YN = "n" ] ; then
			rm -rf mosbuild* 2> /dev/null
		fi
	else
		cancelPrep "** Error: Image write failed"
	fi
}

cleanUp () {
	rm -f mosbuild.properties 2> /dev/null
	rm -f moosimbu.sh 2> /dev/null
	rm -f mosbuild_befor_usb.txt
	rm -f mosbuild_after_usb.txt
}

isConnected () {
	ping -q -w 1 -c 1 `ip r | grep default | cut -d ' ' -f 3` > /dev/null 2> /dev/null && return 0 || return 1
}

connectInternet () {
	if [ ! -z "$SSID" ] ; then
		echo "#########################################" > /etc/wpa_supplicant/wpa_supplicant.conf
		echo "# This file is automatically generated by" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "# the player Network configuration page." >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "#########################################" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "country=$COUNTRY" > /etc/wpa_supplicant/wpa_supplicant.conf
		echo "ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "update_config=1" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "network={" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "ssid=\"$SSID\"" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "scan_ssid=1" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "psk=\"$PSK\"" >> /etc/wpa_supplicant/wpa_supplicant.conf
		echo "}" >> /etc/wpa_supplicant/wpa_supplicant.conf
	fi
	if ! isConnected ; then
		echo "** Connecting to the internet..."
		/bin/systemctl daemon-reload
		/bin/systemctl restart dhcpcd
		echo "** Connected"
	fi
}

modifyRaspbian () {
	echo "** Modify image"
	touch /boot/ssh
	echo "** Enable SSH"

	sed -i "s/init=.*//" /boot/cmdline.txt
	sed -i "s/quiet.*//" /boot/cmdline.txt

	sed -i "s/^/net.ifnames=0 /" /boot/cmdline.txt
	echo "** Enable familiar network interface names"
	mkdir -p $MOSBUILD_DIR
	cp -f mosbuild.properties $MOSBUILD_DIR
	cp -f $0 $MOSBUILD_DIR/moosimbu.sh
	chmod +x $MOSBUILD_DIR/moosimbu.sh
	chown -R 1000.1000 $MOSBUILD_DIR
	echo "** Install main worker script"

	sed -i "s/^exit.*//" /etc/rc.local
	echo "$MOSBUILD_DIR/moosimbu.sh >> /home/pi/mosbuild.log 2>> /home/pi/mosbuild.log" >> /etc/rc.local
	echo "exit 0" >> /etc/rc.local
	echo "** Enable script for autorun after reboot"

	sed -i "s/raspberrypi/$HOSTNAME/" /etc/hostname
	sed -i "s/raspberrypi/$HOSTNAME/" /etc/hosts
	echo "** Change host name to $HOSTNAME"
	STEP_1
	echo "3A" > $MOSBUILD_STEP
	echo "** Cleanup"
	echo "** Reboot"
	sync
	reboot
}

raspbian2moOde (){
	echo "** Would you update this installation to Moode Audio ?"
	readYnInput "** install moOde Audio on the main SDCard (y/n)? "
	if [ $YN = "y" ] ; then
		UPDATE2MOODE="$YN"
		getOptions
		changeHDMI
		connectInternet
		confirmBuild
		testInternet
		dnldHelpers
		updProperties
		loadEnv
		modifyRaspbian
		cleanUp
		echo "Reboot"
		shutdown -r now
		exit 0 # Just for the fun !
	fi
}

##//////////////////////////////////////////////////////////////
##
## MAIN
##
##//////////////////////////////////////////////////////////////
if [ -f $MOSBUILD_PROP ] ; then
	. $MOSBUILD_PROP


	STEP=`cat $MOSBUILD_STEP`
	case $STEP in
		2) 
			STEP_2
			;;
		3A)
			STEP_3A
			;;
		3B-4)
			STEP_3B_4
			;;
		5-6)
			STEP_5_6
			;;
		7-8)
			STEP_7_8
			;;
		9-10)
			STEP_9_10
			;;
		11)
			STEP_11
			;;
		12-13)
			STEP_12_13
			;;
		C1_C9)
			COMP_C1_C9
			;;
		*)
			echo "** Error: should never arrive at case = *"
			;;
	esac

	exit 0
else
	while [[ $# -gt 0 ]]
	do
		key="$1"

		case $key in
			-h|--hdmi)
				HDMI_DRIVE="$2"
				shift
				shift
				;;
		    -u|--update)
				UPDATE2MOODE="y"
				SQUASH_FS="y"
				LATEST_KERNEL="y"
				ADDL_COMPONENTS="y"
				shift
				;;
			-m|--minimum_update)
				UPDATE2MOODE="y"
				shift
				;;				
			*)
				echo "Invalid parameter : $key"
				exit 1
				;;
		esac
	done
	if [ ! -z "$UPDATE2MOODE" ] ; then
		getWirelessInfos
		testInternet
		dnldHelpers
		updProperties
		loadEnv
		modifyRaspbian	
	fi
	mainBanner
	testUnzip
	testDiskSpace
	passNhostname
	raspbian2moOde
	getTargetUsb
	getOptions
	changeHDMI
	confirmBuild
	testInternet
	dnldHelpers
	updProperties
	loadEnv
	# check if img already exist
	if [ -f mosbuild/$RASPBIAN_IMG ] ; then
		echo "** Base OS image was saved from previous build"
		readYnInput "** Use saved image to write to USB drive (y/n)? "
		if [ $YN = "y" ] ; then
			writeImage
			cleanUp
			exit 0
		fi
	fi
	dnldRaspbian
	unzipRaspbian
	mountImage
	modifyImage
	umountImage
	writeImage
	cleanUp
	exit 0
fi
##//////////////////////////////////////////////////////////////
## END
##//////////////////////////////////////////////////////////////
